import React from 'react';
import _ from 'lodash';
import {Card, CardActions, CardHeader, CardMedia, CardTitle, CardText} from 'material-ui/Card';
import {GridList, GridTile} from 'material-ui/GridList';
import MuiThemeProvider from 'material-ui/styles/MuiThemeProvider';
import getMuiTheme from 'material-ui/styles/getMuiTheme';
import {List, ListItem} from 'material-ui/List';
import Avatar from 'material-ui/Avatar';
import {pharse} from '../../utility/mooApp';
import {initLoadMoreBehavior} from '../../utility/mooApp';
import {FirstLoading} from "../utility/FirstLoading";
import {VideoList} from "./LivevideoList";

import RaisedButton from 'material-ui/RaisedButton';
import TextField from 'material-ui/TextField';
import LivevideoActions from '../../data/actions/LivevideoActions';
import AppAction from '../../utility/AppAction';

class BrowseVideo extends React.Component {
    constructor(props) {
        super(props);
        this.handleSearch = this.handleSearch.bind(this);
        this.handleChange = this.handleChange.bind(this);
        this.state = {value:'',isSearch:false,showSearch:false};

        if(window.browseType != 'group') {
            if (_.has(window,"categoryId")) {
                this.props.browseVideoByCategory(window.categoryId);
            }
            else {
                this.props.browseVideoByType(window.browseType);
            }
            initLoadMoreBehavior(function(){
                if(this.state.isSearch) {
                    LivevideoActions.searchNextLivevideo(this.state.value);
                }
                else {
                    if (_.has(window,"categoryId")) {
                        this.props.fetchNextVideoByCategory(window.categoryId);
                    }
                    else {
                        this.props.fetchNextVideos();
                    }
                }
            }.bind(this));

            if (window.browseType == 'all')
            {
                AppAction.showIconSearch();
            }
            window.showSearch = function(){
                this.setState({ showSearch: !this.state.showSearch });
            }.bind(this);
        }
        else {
            this.props.browseVideoGroup(window.groupId);
            initLoadMoreBehavior(function(){
                this.props.fetchNextVideoGroup(window.groupId);
            }.bind(this));
        }
    }

    handleChange(e) {
        this.setState({ value: e.target.value });
    }
    handleSearch() {
        this.setState({ isSearch:true });
        LivevideoActions.searchLivevideo(this.state.value);
    }
    handleClick(url) {
        window.location.href = url;
    }

    render() {
        var content,videoList,createVideo,uploadVideo,canCreate,categoryButton,searchForm;
        searchForm = categoryButton = uploadVideo = createVideo = videoList = content = <div></div>;

        if( this.props.videos.get('isOpen') == true  ) {
            if(window.browseType != 'group') {
                if (!_.has(window,"categoryId")) {
                    //categoryButton = <div onClick={() => this.handleClick(mooConfig.url.base + '/videos/categories_list')} style={{position:"relative",display:"block"}} ><RaisedButton buttonStyle={{height:"auto"}} labelStyle={{textTransform:"uppercase"}} label="Category"  /></div>;
                    if(this.state.showSearch) {
                        searchForm = <div style={{position:"relative"}}>
                            <TextField onChange={this.handleChange} value={this.state.value} underlineStyle={{bottom:"0"}} underlineFocusStyle={{bottom:"0",borderColor:"#247BBA"}} style={{width:"auto",paddingRight:"150px",display:"block",background:"#fff",marginTop:"10px"}} hintText={pharse('searchVideo')} />
                            <div onClick={() => this.handleSearch()} style={{position:"absolute",right:"10px",top:"50%",transform: "translate(0, -50%)",zIndex:"100"}} ><RaisedButton buttonStyle={{height:"auto"}} labelStyle={{textTransform:"uppercase"}} label={pharse('searchVideoBtn')}  /></div>
                        </div>;
                    }
                }
                videoList = <div>{categoryButton}{searchForm}<VideoList {...this.props} ></VideoList></div>;
            }
            else {
                videoList = <VideoList {...this.props} groupId={window.groupId} ></VideoList>;
            }
        }
        else {
            content = <FirstLoading></FirstLoading>;
        }

        return <MuiThemeProvider ><div>{content}{videoList}</div></MuiThemeProvider>;

    }
}
function LivevideoBrowseContent(props) {
    return <BrowseVideo {...props} />
}

export default LivevideoBrowseContent;
